@@ -10,6 +10,7 @@
 #	pragma hdrstop
 #endif
 
+
 void
 FTPDeallocateHost(const FTPCIPtr cip)
 {
@@ -56,7 +57,7 @@ FTPAllocateHost(const FTPCIPtr cip)
 		} else {
 			buf = (char *) calloc((size_t) 1, cip->bufSize);
 			if (buf == NULL) {
-				Error(cip, kDontPerror, "Malloc failed.\n");
+				FTPLogError(cip, kDontPerror, "Malloc failed.\n");
 				cip->errNo = kErrMallocFailed;
 				return (kErrMallocFailed);
 			}
@@ -92,7 +93,7 @@ FTPLoginHost(const FTPCIPtr cip)
 	ResponsePtr rp;
 	int result = kErrLoginFailed;
 	int anonLogin;
-	int sentpass = 0;
+	int sentpass = 0, fwsentpass = 0;
 	int fwloggedin;
 	int firstTime;
 	char cwd[512];
@@ -144,12 +145,18 @@ FTPLoginHost(const FTPCIPtr cip)
 
 		switch (rp->code) {
 			case 220:	/* Welcome, ready for new user. */
-				if ((cip->firewallType == kFirewallNotInUse) || (fwloggedin != 0)) {
+				if ((cip->firewallType == kFirewallNotInUse) || (fwloggedin != 0) || (fwsentpass != 0)) {
 					ReInitResponse(cip, rp);
 					result = RCmd(cip, rp, "USER %s", cip->user);
 				} else if (cip->firewallType == kFirewallUserAtSite) {
 					ReInitResponse(cip, rp);
 					result = RCmd(cip, rp, "USER %s@%s", cip->user, cip->host);
+				} else if (cip->firewallType == kFirewallUserAtSitePort) {
+					ReInitResponse(cip, rp);
+					result = RCmd(cip, rp, "USER %s@%s:%u", cip->user, cip->host, cip->port);
+				} else if (cip->firewallType == kFirewallUserAtSitePort2) {
+					ReInitResponse(cip, rp);
+					result = RCmd(cip, rp, "USER %s@%s %u", cip->user, cip->host, cip->port);
 				} else if (cip->firewallType == kFirewallUserAtUserPassAtPass) {
 					ReInitResponse(cip, rp);
 					result = RCmd(cip, rp, "USER %s@%s@%s", cip->user, cip->firewallUser, cip->host);
@@ -192,7 +199,7 @@ FTPLoginHost(const FTPCIPtr cip)
 					/* only reached when !fwloggedin */
 					ReInitResponse(cip, rp);
 					result = RCmd(cip, rp, "USER %s", cip->user);
-				} else /* kFirewallUserAtSite */ {
+				} else /* kFirewallUserAtSite[Port[2]] */ {
 					goto okay;
 				}
 				break;
@@ -202,31 +209,36 @@ FTPLoginHost(const FTPCIPtr cip)
 				goto done;
 				
 			case 331:	/* 331 User name okay, need password. */
-				if ((cip->firewallType == kFirewallNotInUse) || (fwloggedin != 0)) {
+				if ((cip->firewallType == kFirewallNotInUse) || (fwloggedin != 0) || (fwsentpass != 0)) {
 					if ((cip->pass[0] == '\0') && (cip->passphraseProc != kNoFTPGetPassphraseProc))
 						(*cip->passphraseProc)(cip, &rp->msg, cip->pass, sizeof(cip->pass));
 					ReInitResponse(cip, rp);
 					result = RCmd(cip, rp, "PASS %s", cip->pass);
-				} else if (cip->firewallType == kFirewallUserAtSite) {
+					sentpass++;
+				} else if ((cip->firewallType == kFirewallUserAtSite) || (cip->firewallType == kFirewallUserAtSitePort) || (cip->firewallType == kFirewallUserAtSitePort2)) {
 					ReInitResponse(cip, rp);
 					result = RCmd(cip, rp, "PASS %s", cip->pass);
+					sentpass++;
 				} else if (cip->firewallType == kFirewallUserAtUserPassAtPass) {
 					ReInitResponse(cip, rp);
 					result = RCmd(cip, rp, "PASS %s@%s", cip->pass, cip->firewallPass);
+					fwsentpass++;
 				} else if (cip->firewallType == kFirewallUserAtSiteFwuPassFwp) {
 					ReInitResponse(cip, rp);
 					result = RCmd(cip, rp, "PASS %s", cip->pass);
+					sentpass++;
 				} else if (cip->firewallType == kFirewallFwuAtSiteFwpUserPass) {
 					/* only reached when !fwloggedin */
 					ReInitResponse(cip, rp);
 					result = RCmd(cip, rp, "PASS %s", cip->firewallPass);
+					fwsentpass++;
 				} else if (cip->firewallType > kFirewallNotInUse) {
 					ReInitResponse(cip, rp);
 					result = RCmd(cip, rp, "PASS %s", cip->firewallPass);
+					fwsentpass++;
 				} else {
 					goto unknown;
 				}
-				sentpass++;
 				break;
 
 			case 332:	/* 332 Need account for login. */
@@ -247,9 +259,9 @@ FTPLoginHost(const FTPCIPtr cip)
 			default:
 			unknown:
 				if (rp->msg.first == NULL) {
-					Error(cip, kDontPerror, "Lost connection during login.\n");
+					FTPLogError(cip, kDontPerror, "Lost connection during login.\n");
 				} else {
-					Error(cip, kDontPerror, "Unexpected response: %s\n",
+					FTPLogError(cip, kDontPerror, "Unexpected response: %s\n",
 						rp->msg.first->line
 					);
 				}
@@ -291,6 +303,8 @@ okay:
 
 	if (result < 0)
 		cip->errNo = result;
+	else
+		(void) gettimeofday(&cip->loginTime, NULL);
 	return result;
 
 done:
@@ -313,6 +327,8 @@ done2:
 	}
 	if (result < 0)
 		cip->errNo = result;
+	else
+		(void) gettimeofday(&cip->loginTime, NULL);
 	return result;
 }	/* FTPLoginHost */
 
@@ -368,7 +384,7 @@ FTPQueryFeatures(const FTPCIPtr cip)
 {
 	ResponsePtr rp;
 	int result;
-	LinePtr lp;
+	FTPLinePtr lp;
 	char *cp, *p;
 
 	if (cip == NULL)
@@ -386,10 +402,11 @@ FTPQueryFeatures(const FTPCIPtr cip)
 		cip->hasMDTM = kCommandNotAvailable;
 		cip->hasREST = kCommandNotAvailable;
 		cip->NLSTfileParamWorks = kCommandAvailable;
-		cip->hasUTIME = kCommandNotAvailable;
 		cip->hasCLNT = kCommandNotAvailable;
 		cip->hasMLST = kCommandNotAvailable;
 		cip->hasMLSD = kCommandNotAvailable;
+		cip->hasSITE_UTIME = kCommandNotAvailable;
+		cip->hasHELP_SITE = kCommandNotAvailable;
 		return (kNoErr);
 	}
 
@@ -452,7 +469,7 @@ FTPQueryFeatures(const FTPCIPtr cip)
 				} else if (ISTRNCMP(cp, "REST", 4) == 0) {
 					cip->hasREST = kCommandAvailable;
 				} else if (ISTRNCMP(cp, "UTIME", 5) == 0) {
-					cip->hasUTIME = kCommandAvailable;
+					cip->hasSITE_UTIME = kCommandAvailable;
 				} else if (ISTRNCMP(cp, "MLST", 4) == 0) {
 					cip->hasMLST = kCommandAvailable;
 					cip->hasMLSD = kCommandAvailable;
@@ -470,15 +487,20 @@ FTPQueryFeatures(const FTPCIPtr cip)
 			}
 		}
 
+		/* You can set cip->hasHELP_SITE to kCommandNotAvailable
+		 * if your host chokes (i.e. IBM Mainframes) when you
+		 * do "HELP SITE".
+		 */
 		ReInitResponse(cip, rp);
-		result = RCmd(cip, rp, "HELP SITE");
+		result = (cip->hasHELP_SITE == kCommandNotAvailable) ? (-1) : RCmd(cip, rp, "HELP SITE");
 		if (result == 2) {
+			cip->hasHELP_SITE = kCommandAvailable;
 			for (lp = rp->msg.first; lp != NULL; lp = lp->next) {
 				cp = lp->line;
 				if (strstr(cp, "RETRBUFSIZE") != NULL)
-					cip->hasRETRBUFSIZE = kCommandAvailable;
+					cip->hasSITE_RETRBUFSIZE = kCommandAvailable;
 				if (strstr(cp, "RBUFSZ") != NULL)
-					cip->hasRBUFSZ = kCommandAvailable;
+					cip->hasSITE_RBUFSZ = kCommandAvailable;
 				/* See if RBUFSIZ matches (but not STORBUFSIZE) */
 				if (
 					((p = strstr(cp, "RBUFSIZ")) != NULL) &&
@@ -487,15 +509,15 @@ FTPQueryFeatures(const FTPCIPtr cip)
 						((p > cp) && (!isupper((int) p[-1])))
 					)
 				)
-					cip->hasRBUFSIZ = kCommandAvailable;
+					cip->hasSITE_RBUFSIZ = kCommandAvailable;
 				if (strstr(cp, "STORBUFSIZE") != NULL)
-					cip->hasSTORBUFSIZE = kCommandAvailable;
+					cip->hasSITE_STORBUFSIZE = kCommandAvailable;
 				if (strstr(cp, "SBUFSIZ") != NULL)
-					cip->hasSBUFSIZ = kCommandAvailable;
+					cip->hasSITE_SBUFSIZ = kCommandAvailable;
 				if (strstr(cp, "SBUFSZ") != NULL)
-					cip->hasSBUFSZ = kCommandAvailable;
+					cip->hasSITE_SBUFSZ = kCommandAvailable;
 				if (strstr(cp, "BUFSIZE") != NULL)
-					cip->hasBUFSIZE = kCommandAvailable;
+					cip->hasSITE_BUFSIZE = kCommandAvailable;
 			}
 		}
 		DoneWithResponse(cip, rp);
@@ -541,19 +563,130 @@ FTPCloseHost(const FTPCIPtr cip)
 	 * if you OpenHost with this again.
 	 */
 	FTPDeallocateHost(cip);
+
+	if (cip->disconnectTime.tv_sec == 0)
+		(void) gettimeofday(&cip->disconnectTime, NULL);
 	return (result);
 }	/* FTPCloseHost */
 
 
 
 
+void
+FTPInitialLogEntry(const FTPCIPtr cip)
+{
+#if defined(HAVE_UNAME) && defined(HAVE_SYS_UTSNAME_H)
+	struct utsname u;
+#endif
+
+	if (cip->startTime.tv_sec == 0) {
+		(void) gettimeofday(&cip->startTime, NULL);
+
+		/* Log some headers for debugging.
+		 *
+		 * Note that this is the soonest we can do this,
+		 * since the user may not have set the debugLog
+		 * fields right away.
+		 */
+		PrintF(cip, "%s compiled for %s\n",
+			gLibNcFTPVersion + 5,
+#ifdef O_S
+			O_S
+#elif (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
+			"Windows"
+#elif defined(__CYGWIN__)
+			"Cygwin"
+#else
+			"UNIX"
+#endif
+		);
+	
+#if defined(HAVE_UNAME) && defined(HAVE_SYS_UTSNAME_H)
+		memset(&u, 0, sizeof(u));
+		if (uname(&u) == 0) {
+			PrintF(cip, "Uname: %s|%s|%s|%s|%s\n",
+				u.sysname,
+				u.nodename,
+				u.release,
+				u.version,
+				u.machine
+			);
+		}
+#endif
+
+#if defined(SOLARIS) || defined(AIX)
+		{
+			FILE *fp;
+			char line[256], *cp;
+
+#if defined(SOLARIS)
+			fp = fopen("/etc/release", "r");
+#elif defined(AIX)
+			fp = fopen("/usr/lpp/bos/aix_release.level", "r");
+#endif
+			if (fp != NULL) {
+				memset(line, 0, sizeof(line));
+				(void) fgets(line, sizeof(line) - 1, fp);
+				cp = line + strlen(line) - 1;
+				while (cp > line) {
+					if (! isspace((int) *cp))
+						break;
+					--cp;
+				}
+				cp[1] = '\0';
+				cp = line;
+				while ((*cp != '\0') && (isspace((int) *cp)))
+					cp++;
+				fclose(fp);
+				PrintF(cip, "Release: %s\n", cp);
+			}
+		}
+#endif
+
+
+#if defined(HAVE_SYSINFO) && defined(SI_ARCHITECTURE) && defined(SI_ISALIST) && defined(SI_PLATFORM)
+		{
+			char si_platform[64];
+			char si_arch[32];
+			char si_isalist[256];
+
+			memset(si_platform, 0, sizeof(si_platform));
+			memset(si_arch, 0, sizeof(si_arch));
+			memset(si_isalist, 0, sizeof(si_isalist));
+
+			(void) sysinfo(SI_PLATFORM, si_platform, sizeof(si_platform) - 1);
+			(void) sysinfo(SI_ARCHITECTURE, si_arch, sizeof(si_arch) - 1);
+			(void) sysinfo(SI_ISALIST, si_isalist, sizeof(si_isalist) - 1);
+			PrintF(cip, "Sysinfo: %s|%s|%s\n",
+				si_platform,
+				si_arch,
+				si_isalist
+			);
+		}
+#endif
+
+#if defined(HAVE_GNU_GET_LIBC_VERSION) && defined(HAVE_GNU_GET_LIBC_RELEASE)
+		PrintF(cip, "Glibc: %s (%s)\n",
+			gnu_get_libc_version(),
+			gnu_get_libc_release()
+		);
+#endif
+	} else {
+		/* Starting a new set of opens, but simply update
+		 * the timestamp rather than re-log the headers.
+		 */
+		(void) gettimeofday(&cip->startTime, NULL);
+	}
+}	/* FTPInitialLogEntry */
+
+
+
 int
 FTPOpenHost(const FTPCIPtr cip)
 {
 	int result;
 	time_t t0, t1;
 	int elapsed;
-	int dials;
 
 	if (cip == NULL)
 		return (kErrBadParameter);
@@ -565,9 +698,11 @@ FTPOpenHost(const FTPCIPtr cip)
 		return (kErrBadParameter);
 	}
 
-	for (	result = kErrConnectMiscErr, dials = 0;
-		cip->maxDials < 0 || dials < cip->maxDials;
-		dials++)
+	FTPInitialLogEntry(cip);
+
+	for (	result = kErrConnectMiscErr, cip->numDials = 0;
+		cip->maxDials < 0 || cip->numDials < cip->maxDials;
+	)	
 	{
 		/* Allocate (or if the host was closed, reallocate)
 		 * the transfer data buffer.
@@ -576,15 +711,22 @@ FTPOpenHost(const FTPCIPtr cip)
 		if (result < 0)
 			return (result);
 
-		if (dials > 0)
-			PrintF(cip, "Retry Number: %d\n", dials);
+		memset(&cip->connectTime, 0, sizeof(cip->connectTime));
+		memset(&cip->loginTime, 0, sizeof(cip->loginTime));
+		memset(&cip->disconnectTime, 0, sizeof(cip->disconnectTime));
+
+		cip->totalDials++;
+		cip->numDials++;
+		if (cip->numDials > 1)
+			PrintF(cip, "Retry Number: %d\n", cip->numDials - 1);
 		if (cip->redialStatusProc != 0)
-			(*cip->redialStatusProc)(cip, kRedialStatusDialing, dials);
+			(*cip->redialStatusProc)(cip, kRedialStatusDialing, cip->numDials - 1);
 		(void) time(&t0);
 		result = OpenControlConnection(cip, cip->host, cip->port);
 		(void) time(&t1);
 		if (result == kNoErr) {
 			/* We were hooked up successfully. */
+			(void) gettimeofday(&cip->connectTime, NULL);
 			PrintF(cip, "Connected to %s.\n", cip->host);
 
 			result = FTPLoginHost(cip);
@@ -621,7 +763,7 @@ FTPOpenHost(const FTPCIPtr cip)
 		/* Retryable error, wait and then redial. */
 		if (cip->redialDelay > 0) {
 			/* But don't sleep if this is the last loop. */
-			if ((cip->maxDials < 0) || (dials < (cip->maxDials - 1))) {
+			if ((cip->maxDials < 0) || (cip->numDials < (cip->maxDials))) {
 				elapsed = (int) (t1 - t0);
 				if (elapsed < cip->redialDelay) {
 					PrintF(cip, "Sleeping %u seconds.\n",
@@ -681,24 +823,27 @@ FTPInitConnectionInfo2(const FTPLIPtr lip, const FTPCIPtr cip, char *const buf,
 	cip->hasMDTM = kCommandAvailabilityUnknown;
 	cip->hasREST = kCommandAvailabilityUnknown;
 	cip->hasNLST_d = kCommandAvailabilityUnknown;
-	cip->hasUTIME = kCommandAvailabilityUnknown;
 	cip->hasFEAT = kCommandAvailabilityUnknown;
 	cip->hasMLSD = kCommandAvailabilityUnknown;
 	cip->hasMLST = kCommandAvailabilityUnknown;
 	cip->hasCLNT = kCommandAvailabilityUnknown;
-	cip->hasRETRBUFSIZE = kCommandAvailabilityUnknown;
-	cip->hasRBUFSIZ = kCommandAvailabilityUnknown;
-	cip->hasRBUFSZ = kCommandAvailabilityUnknown;
-	cip->hasSTORBUFSIZE = kCommandAvailabilityUnknown;
-	cip->hasSBUFSIZ = kCommandAvailabilityUnknown;
-	cip->hasSBUFSZ = kCommandAvailabilityUnknown;
+	cip->hasHELP_SITE = kCommandAvailabilityUnknown;
+	cip->hasSITE_UTIME = kCommandAvailabilityUnknown;
+	cip->hasSITE_RETRBUFSIZE = kCommandAvailabilityUnknown;
+	cip->hasSITE_RBUFSIZ = kCommandAvailabilityUnknown;
+	cip->hasSITE_RBUFSZ = kCommandAvailabilityUnknown;
+	cip->hasSITE_STORBUFSIZE = kCommandAvailabilityUnknown;
+	cip->hasSITE_SBUFSIZ = kCommandAvailabilityUnknown;
+	cip->hasSITE_SBUFSZ = kCommandAvailabilityUnknown;
 	cip->STATfileParamWorks = kCommandAvailabilityUnknown;
 	cip->NLSTfileParamWorks = kCommandAvailabilityUnknown;
 	cip->firewallType = kFirewallNotInUse;
 	cip->startingWorkingDirectory = NULL;
 	cip->shutdownUnusedSideOfSockets = 0;
+	InitLineList(&cip->lastFTPCmdResultLL);
 	(void) STRNCPY(cip->magic, kLibraryMagic);
 	(void) STRNCPY(cip->user, "anonymous");
+	(void) gettimeofday(&cip->initTime, NULL);
 	return (kNoErr);
 }	/* FTPInitConnectionInfo2 */
 
