@@ -6,7 +6,7 @@
  */
 
 #define kSpoolDir "spool"
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	define kSpoolLog "log.txt"
 #else
 #	define kSpoolLog "log"
